package org.tbasket.handler

import org.tbasket.api.compute.APIRequestHandler
import org.tbasket.db.Database.ctx._
import org.tbasket.db.schemas.User
import org.tbasket.session.UserSessionHandler
import zio.ZIO
import zio.http.model.{Cookie, Headers, Status}
import zio.http.{Request, Response}
import zio.json._
import zio.json.ast.Json.Str
import zio.json.ast.{Json, JsonCursor}

class LoginHandler(sessions: UserSessionHandler) extends APIRequestHandler {
    
    private def getUser(json: Json): Either[String, User] = {
        (for {
            mail <- json.get[Str](JsonCursor.field("mail").isString).map(_.value)
            password <- json.get[Str](JsonCursor.field("password").isString).map(_.value)
        } yield quote { //                                          TODO use argon2id
            User.schema.filter(usr => usr.mailAddress == mail && usr.passwordHash == password.hashCode)
        }).flatMap(_.nested.value.toRight("password or email incorrect"))
    }
    
    override def post: ZIO[Request, Throwable, Response] = ZIO.serviceWithZIO[Request] { r =>
        r.body.asString
            .map(_.fromJson[Json].flatMap(getUser) match {
                case Left(err)   => //if account not found or password mismatches
                    Response.json(Json.Obj("error" -> Json.Str(err)).toJson)
                case Right(user) =>
                    val session = sessions.bind(user)
                    
                    Response(
                        status = Status.Found,
                        headers = Headers.location("/") ++ Headers.setCookie(
                            Cookie(
                                name = "token",
                                content = session.token.toString,
                                maxAge = Some(session.expirationDate)
                            ))
                    )
            }
            )
    }
}
