package org.tbasket.session

import org.tbasket.db.schemas.User
import org.tbasket.session.UserSessionHandler.SessionLifespan

import java.time.{Duration, Instant}
import java.util.UUID
import scala.collection.mutable

object UserSessionHandler {
    final val SessionLifespan = Duration.ofDays(31)
}

class UserSessionHandler {
    
    private val sessionsUser = mutable.HashMap.empty[User, UserSession]
    private val sessionsToken = mutable.HashMap.empty[UUID, UserSession]
    
    def findSession(token: UUID): Option[UserSession] = {
        sessionsToken.get(token) match {
            case Some(session) if System.currentTimeMillis() > session.expirationDate =>
                unbind(token) //session has expired, unbind session
                None
            
            case s => s
        }
    }
    
    private def unbind(token: UUID): Unit = {
        sessionsToken.remove(token)
            .foreach(s => sessionsUser.remove(s.user))
    }
    
    private def unbind(user: User): Unit = {
        sessionsUser.remove(user)
            .foreach(s => sessionsToken.remove(s.token))
    }
    
    def bind(user: User): UserSession = {
        if (sessionsUser.contains(user)) {
            //if there was a session already bound, then remove it
            unbind(user)
        }
        val uuid    = UUID.randomUUID()
        val limit   = System.currentTimeMillis() + Instant.now().plus(SessionLifespan).toEpochMilli
        val session = UserSession(user, uuid, limit)
        sessionsUser.put(user, session)
        sessionsToken.put(uuid, session)
        session
    }
    
}